// Grammatica ANTLR per il linguaggio ALCN
// Autore: Daniele Scrimieri
// Data: 21-06-2005

// specifica il package al quale appartengono le classi generate
header {
    package it.uniba.di.alcn;
}

class AlcnParser extends Parser;

// disabilita la gestione di default degli errori sintattici
options {
    defaultErrorHandler=false;
}

// ritorna un concetto ALCN
concept
    returns [Concept c]
    {
        Concept c1;
        ConceptSet cs;
        c = null;
    }
    :   i1:ID                            { c = AtomicConcept.create(i1.getText()); }
    |   LPAREN
        (   "not" c1=concept             { c = NegatedConceptDescription.create(c1); }
        |   "and" cs=concept_list        { c = AndConceptDescription.create(cs); }
        |   "or" cs=concept_list         { c = OrConceptDescription.create(cs); }        
        |   "some" i2:ID c1=concept      { c = SomeConceptDescription.create(AtomicRole.create(i2.getText()), c1); }
        |   "all" i3:ID c1=concept       { c = AllConceptDescription.create(AtomicRole.create(i3.getText()), c1); }
        |   ATLEAST int1:INT id1:ID      { c = AtLeastConceptDescription.create(Integer.valueOf(int1.getText()), AtomicRole.create(id1.getText())); }
        |   ATMOST int2:INT id2:ID       { c = AtMostConceptDescription.create(Integer.valueOf(int2.getText()), AtomicRole.create(id2.getText())); }
        )
        RPAREN
    ;

// ritorna un insieme di concetti ALCN
concept_list
    returns [ConceptSet cs]
    {
        Concept c;
        ConceptSet cs1;
        cs = new ConceptSet();
    }
    :   (     c=concept                  { cs.add(c); }      
              cs1=concept_list           { cs.addAll(cs1); }
        )?
    ;

class AlcnLexer extends Lexer;

options {
    k = 4;                  // 4 caratteri di lookahead
    caseSensitive=false;    // case insensitive
    caseSensitiveLiterals=false;
}

ATLEAST : "at-least" ;
ATMOST  : "at-most"  ;

LPAREN  options { paraphrase = "'('"; }     :   '(' ;
RPAREN  options { paraphrase = "')'"; }     :   ')' ;

protected
UNDERSCORE  :   '_' ;

protected
DIGIT
    :   '0'..'9'
    ;

protected
LETTER
    :   ('a'..'z' | 'A'..'Z')
    ;

INT
    options { paraphrase = "a non negative integer"; }
    :   (DIGIT)+
    ;

ID
    options { paraphrase = "an identifier"; }
    :   (LETTER | UNDERSCORE)
        (LETTER | UNDERSCORE | DIGIT)*
    |   "*bottom*"
    |   "*top*"
    ;

protected
NL
    :	(   options { generateAmbigWarnings = false; }
        :   "\r\n"  // Evil DOS
        |	'\r'    // Macintosh
        |	'\n'    // Unix (the right way)
        )
        { newline(); }
    ;

WS
	:	(	' '
		|	'\t'
		|	'\f'
        |   NL
		)+
		{ $setType(Token.SKIP); }
	;

// commenti su una linea 
SL_COMMENT
	:	";"
		(~('\n' | '\r'))*
        (NL)?
		{ $setType(Token.SKIP); }
	;

// commenti su più linee
ML_COMMENT
	:	"/*"
		(   options { generateAmbigWarnings = false; }
        :   { LA(2) != '/' }? '*'
		|   NL
		|	~('*' | '\n' | '\r')
		)*
		"*/"
		{ $setType(Token.SKIP); }
	;
